#!/bin/bash

bold=$(tput bold 2>/dev/null)
sgr0=$(tput sgr0 2>/dev/null)

username=$(whoami)
ports=()

red_echo() {
    echo -e "\e[31m$1\e[0m"
}

green_echo() {
    echo -e "\e[32m$1\e[0m"
}

yellow_echo() {
    echo -e "\e[33m$1\e[0m"
}


get_local_ips() {
    local ips=""

    if command -v ip &>/dev/null; then
        ips=$(ip addr 2>/dev/null | grep -w inet | awk '{print $2}' | cut -d/ -f1 | grep -v '^127\.')
    fi

    if [ -z "$ips" ] && command -v ifconfig &>/dev/null; then
        ips=$(ifconfig 2>/dev/null | grep -w inet | awk '{print $2}' | grep -v '^127\.')
    fi

    if [ -z "$ips" ] && command -v hostname &>/dev/null; then
        ips=$(hostname -I 2>/dev/null | tr ' ' '\n')
    fi

    if [ -z "$ips" ]; then
        ips="127.0.0.1"
    fi

    echo "${ips}"
}

# input ip
while true; do
    local_ips=$(get_local_ips)
    default_ip=$(echo "$local_ips" | head -n 1)

    read -p "Enter the IP (Default: ${default_ip}) (only support local IP): " ip
    ip=${ip:-${default_ip}}
    flag=0
    if [[ ! $ip =~ ^[0-9]+\.[0-9]+\.[0-9]+\.[0-9]+$ ]]; then
        red_echo "${bold} Invalid IP address format. Please enter a valid IP address.${sgr0}"
        flag=1
    else
        flag=1
        if [[ $ip == "127.0.0.1" ]]; then
            green_echo "${bold} Valid local IP: $ip${sgr0}"
            flag=0
            break
        fi
        for local_ip in $local_ips; do
            if [[ $local_ip == "$ip" ]]; then
                green_echo "${bold} Valid local IP: $ip${sgr0}"
                flag=0
                break
            fi
        done

        if [[ $flag -eq 1 ]]; then
            red_echo "${bold} The IP $ip is not a local IP.${sgr0}"
        fi
    fi

    if [[ $flag -eq 0 ]]; then
        break
    fi
done

# input user password
while true; do
    read -sp "Enter the current user password : " user_password
    sudo -k
    if echo ${user_password} | sudo -S ls  &> /dev/null; then
        break
    else
        red_echo "${bold} Insufficient sudo privileges. Please use a user with sudo privileges or execute the script again with the correct privileges.${sgr0}"
    fi
done
echo ""

obd env set HOST_IP_MODE 1

obd host precheck ${username} $ip code100
if [[ $? -eq 100 ]]; then
    while true; do
        read -p "Do you want to modify the parameters above? (y/n) (Default: y)" confirm4
        confirm4=${confirm4:-y}
        if [[ $confirm4 == "y" || $confirm4 == "Y" ]]; then
            obd host init ${username} $ip code101
            if [[ $? -eq 101 ]]; then
                read -p " Do you want to quit the script and manually restart the machine to apply the optimizations? (y/n) (Default: n)" confirm5
                confirm5=${confirm5:-n}
                if [[ $confirm5 == "y" || $confirm4 == "Y" ]]; then
                    exit 0
                fi
            fi
            break
        elif [[ $confirm4 == "n" || $confirm4 == "N" ]]; then
            break
        else
            echo "${bold} Invalid input. Please enter 'y' or 'n'. ${sgr0}"
            continue
        fi
    done
fi


function port_check {
    port="$1"
    hex_port=$(printf "%04X" $port)
    socket_inodes=$(cat /proc/net/{tcp*,udp*} | grep ":$hex_port" | awk -F' ' '{print $3}' | uniq)
    for use_port in "${ports[@]}"; do
        if [ "$port" -eq "$use_port" ] 2>/dev/null; then
            red_echo " The port $port has already been specified above."
            return 1
        fi
    done
    if [ -z "$socket_inodes" ]; then
        ports+=($port)
        return 0
    else
        red_echo "${bold} The port is already in use. Please enter a different one.  ${sgr0}"
        return 1
    fi
}


function dir_check {
    dir="$1"
    flag=0

    if [[ $dir != /* ]]; then
        red_echo "${bold} The directory must be an absolute path. Please try again. ${sgr0}"
        flag=1
    fi

    if [ -d "$dir" ]; then
        if [ "$(ls -A "$dir")" ]; then
            red_echo "${bold} The directory must be empty. Please choose an empty directory. ${sgr0}"
            flag=1
        fi

        if [ ! -r "$dir" ] || [ ! -w "$dir" ]; then
            red_echo "${bold} The current user doesn't have read/write permissions for this directory. Please choose a directory with the required access permissions. ${sgr0}"
            flag=1
        fi
    else
        parent_dir1=$(dirname "$dir")
        while true; do
            if [ ! -d "$parent_dir1" ]; then
                parent_dir1=$(dirname "$parent_dir1")
            elif [ ! -r "$parent_dir1" ] || [ ! -w "$parent_dir1" ]; then
                red_echo "${bold} The current user doesn't have read/write permissions for the parent directory $parent_dir1. Please choose a directory with the required access permissions. ${sgr0}"
                flag=1
                break
            else
                break
            fi
        done
    fi

    if [[ "$2" == "true" ]]; then
        mkdir -p $dir
        disk_available=$(df $dir | awk 'NR==2 {print $4}')
        if awk -v disk="$disk_available" -v mem="$memory_limit" 'BEGIN { exit !(disk < 3 * mem) }'; then
            yellow_echo "${bold}  Insufficient disk space in the directory. For business purposes, OceanBase recommends that disk space should not be less than 3*memory limit (${memory_limit}). ${sgr0}"
        fi
        if awk -v disk="$disk_available" -v mem="$memory_limit" 'BEGIN { exit !(disk < 2 * mem) }'; then
            red_echo "${bold}  Insufficient disk space in the directory. Please choose a directory with disk space greater than 2*memory limit (${memory_limit}). ${sgr0}"
            flag=1
        fi
    fi

    if [[ $flag -eq 1 ]]; then
        return 1
    else
        return 0
    fi
}

get_system_memory() {
    local memory_limit=$1
    local system_memory
    if [ $memory_limit -lt 12 ]; then
        system_memory=1
    elif [ $memory_limit -lt 20 ]; then
        system_memory=5
    elif [ $memory_limit -lt 40 ]; then
        system_memory=6
    elif [ $memory_limit -lt 60 ]; then
        system_memory=7
    elif [ $memory_limit -lt 80 ]; then
        system_memory=8
    elif [ $memory_limit -lt 100 ]; then
        system_memory=9
    elif [ $memory_limit -lt 130 ]; then
        system_memory=10
    else
        system_memory=$(awk -v mem="$memory_limit" 'BEGIN { print int(mem * 0.08) + (mem * 0.08 > int(mem * 0.08) ? 1 : 0) }')
    fi

    echo $system_memory
}

get_sys_cpu() {
    local sys_cpu
    local cpu_count=$1

    if [ $cpu_count -lt 8 ]; then
        sys_cpu=1
    elif [ $cpu_count -lt 16 ]; then
        sys_cpu=2
    elif [ $cpu_count -lt 32 ]; then
        sys_cpu=3
    else
        sys_cpu=4
    fi
    echo $sys_cpu
}

get_sys_log_disk_size() {
    local sys_log_disk_size
    local memory_limit=$1
    if [ $memory_limit -lt 12 ]; then
        sys_log_disk_size=2
    elif [ $memory_limit -lt 40 ]; then
        sys_log_disk_size=3
    elif [ $memory_limit -lt 80 ]; then
        sys_log_disk_size=4
    else
        sys_log_disk_size=5
    fi
    echo $sys_log_disk_size
}

compare_size() {
    local size1=$1
    local size2=$2
    size1=$(echo "$size1" | tr -cd '0-9.')
    size2=$(echo "$size2" | tr -cd '0-9.')
    if awk -v s1="$size1" -v s2="$size2" 'BEGIN { exit !(s1 > s2) }'; then
        return 0
    else
        return 1
    fi
}

format_type() {
    local num=$1
    if [[ $num =~ ^[0-9.]+$ ]]; then
        num=$(printf "%.0f" "$num")
        echo "$num"G
    else
        echo "$num"
    fi
}

same_disk_check() {
    local disk1=$1
    local disk2=$2
    mkdir -p ${disk1}
    mkdir -p ${disk2}
    fs1=$(df $disk1 | awk 'NR==2 {print $1}')
    fs2=$(df $disk2 | awk 'NR==2 {print $1}')
    if [ "$fs1" == "$fs2" ]; then
        return 0
    else
        return 1
    fi
}

generate_root_password() {
    chars='A-Za-z0-9'

    if [ -e /dev/urandom ]; then
        password=$(head /dev/urandom | tr -dc "$chars" | head -c 20)
    else
        password=$(date +%s | sha256sum | base64 | head -c 20)
    fi

    echo $password
}

echo ""

# input cluster name
while true; do
    read -p "Enter the cluster name (Default: standalone; allowed characters: letters, numbers, and underscores): " cluster_name
    cluster_name=${cluster_name:-standalone}
    flag=0
    if [[ ! "$cluster_name" =~ ^[a-zA-Z0-9_]+$ ]]; then
        red_echo "${bold} Invalid cluster name. Only letters, numbers, and underscores are allowed. ${sgr0}"
        continue
    fi

    meta=~/.obd/cluster/${cluster_name}/.data
    if [ -f ${meta} ]; then
        if [[ `grep STATUS_STOPPED ${meta}` ]] || [[ `grep STATUS_RUNNING ${meta}` ]] || [[ `grep STATUS_DEPLOYED ${meta}` ]]; then
            red_echo "${bold} The cluster name already exists and is in a deployed state. Please choose a different name. ${sgr0}"
            flag=1
        fi
    fi

    if [[ $flag -ne 1 ]]; then
        break
    fi
done

# input mysql port
while true; do
    read -p "Enter the OB SQL port (Default: 2881): " mysql_port
    mysql_port=${mysql_port:-2881}
    if port_check ${mysql_port}; then
        break
    fi
done


# input rpc port
while true; do
    read -p "Enter the OB RPC port (Default: 2882): " rpc_port
    rpc_port=${rpc_port:-2882}
    if port_check ${rpc_port}; then
        break
    fi
done

# input obshell port
while true; do
    read -p "Enter the obshell port (Default: 2886): " obshell_port
    obshell_port=${obshell_port:-2886}
    if port_check ${obshell_port}; then
        break
    fi
done

# input root password
while true; do
    default_root_password=$(generate_root_password)
    read -sp "Enter the OB root password (Default: ${default_root_password}): " root_password1
    echo ""
    root_password1=${root_password1:-${default_root_password}}
    if [ $root_password1 == $default_root_password ]; then
        root_password=$root_password1
        break
    fi
    read -sp "Confirm the OB root password: " root_password2
    echo ""
    if [ $root_password1 == $root_password2 ]; then
        root_password=$root_password1
    else
        red_echo "${bold} The two passwords do not match. Please try again. ${sgr0}"
        continue
    fi
    break
done

# input cpu count
while true; do
    default_cpu_count=$(grep -e 'processor\s*:' /proc/cpuinfo | wc -l)
    read -p "Enter the OB cpu count (Default: ${default_cpu_count}): " cpu_count
    cpu_count=${cpu_count:-"${default_cpu_count}"}
    if [[ ! "$cpu_count" =~ ^[0-9]+$ ]] || [ "$cpu_count" -lt 0 ]; then
        red_echo "${bold} Invalid CPU count. Please try again. ${sgr0}"
    elif compare_size 8 ${cpu_count}; then
        cpu_count=8
        yellow_echo "${bold}  The cpu_count cannot be less than 8. It will be set to 8 automatically. ${sgr0}"
        break
    else
        break
    fi
done


# input memory limit
while true; do
    default_memory_limit=$(free -g | grep Mem | awk '{print $7}')
    if [ $default_memory_limit -lt 8 ]; then
        default_memory_limit=${default_memory_limit}
    else
        default_memory_limit=$(awk -v mem="$default_memory_limit" 'BEGIN { printf "%d\n", mem - 1 }')
    fi
    default_memory_limit=$(echo "$default_memory_limit" | tr -cd '0-9.')
    default_memory_limit=$(printf "%.0f" "$default_memory_limit")
    read -p "Enter the OB memory limit (Configurable Range[6, ${default_memory_limit}]， Default: ${default_memory_limit}, Unit: G): " memory_limit
    memory_limit=${memory_limit:-"${default_memory_limit}"}
    if [ "$memory_limit" -lt 0 ]; then
        red_echo "${bold} Invalid memory limit. Please try again. ${sgr0}"
        continue
    fi
    if compare_size 6 $default_memory_limit; then
        red_echo "${bold} The machine's minimum memory cannot be less than min_value (6G). Please try again. ${sgr0}"
        exit 1
    fi
    if compare_size ${memory_limit} ${default_memory_limit}; then
        red_echo "${bold} The memory limit for OB cannot exceed ${default_memory_limit}G. Please try again. ${sgr0}"
    elif compare_size 6 ${memory_limit}; then
        red_echo "${bold} The memory limit for OB cannot be less than 6G. Please try again. ${sgr0}"
    else
        memory_limit=$(echo "$memory_limit" | tr -cd '0-9.')
        break
    fi
done

# input home path
while true; do
    if [ "${username}" = "root" ]; then
        default_home_path="/${username}/${cluster_name}"
    else
        default_home_path="/home/${username}/${cluster_name}"
    fi
    read -p "Enter the OB installation directory (Default: ${default_home_path}): " home_path
    home_path=${home_path:-"${default_home_path}"}
    if dir_check ${home_path} false; then
        break
    fi
done

# input data dir
while true; do
    read -p "Enter the OB data directory (Default: /data/1/${cluster_name}): " data_dir
    data_dir=${data_dir:-"/data/1/${cluster_name}"}
    if dir_check ${data_dir} true; then
        break
    fi
done

# input log dir
while true; do
    read -p "Enter the OB log directory (Default: /data/log1/${cluster_name}): " redo_dir
    redo_dir=${redo_dir:-"/data/log1/${cluster_name}"}
    if dir_check ${redo_dir} true; then
        break
    fi
done

system_memory=$(get_system_memory $memory_limit)
disk_free=$(df -BG ${data_dir} | awk 'NR==2 {print $4}')
all_disk_free=$(echo "$disk_free" | tr -cd '0-9.')
disk_free=$(awk -v df="$all_disk_free" 'BEGIN { print df * 0.9 }')
disk_free=$(echo "$disk_free" | tr -cd '0-9.')
default_log_disk_size=$(awk -v mem="$memory_limit" -v sys_mem="$system_memory" 'BEGIN { print mem * 3 - sys_mem * 2 }')

if same_disk_check ${data_dir} ${redo_dir}; then
    yellow_echo "${bold} The data_dir and redo_dir are using the same disk. ${sgr0}"
    same_disk=1
    if same_disk_check ${home_path} ${redo_dir}; then
        log_size=4
    else
        log_size=0
    fi
    SLOG_SIZE=4
    disk_free=$(awk -v df="$disk_free" -v log1="$log_size" -v slog="$SLOG_SIZE" 'BEGIN { print df - log1 - slog }')
else
    same_disk=0
fi

# input datafile maxsize
while true; do
    flag=0
    double_memory_limit=$(awk -v a="$memory_limit" 'BEGIN{print a*2}')
    if [ $same_disk -eq 1 ]; then
        max_datafile_maxsize=$(awk -v df="$disk_free" -v log_disk="$default_log_disk_size" 'BEGIN { print df - log_disk }')
        max_datafile_maxsize=$(printf "%.0f" "$max_datafile_maxsize")
        if [ "$max_datafile_maxsize" -lt 0 ]; then
            red_echo "${bold} The available disk space is not enough. Please try again. ${sgr0}"
            exit
        fi
        read -p "Enter the OB datafile maxsize (Configurable Range[${double_memory_limit}, ${max_datafile_maxsize}], Default: ${max_datafile_maxsize}, Unit: G): " datafile_maxsize
        datafile_maxsize=${datafile_maxsize:-"${max_datafile_maxsize}"}
        if [ "$datafile_maxsize" -lt 0 ]; then
            red_echo "${bold} Invalid datafile maxsize. Please try again. ${sgr0}"
            continue
        fi
        if compare_size $double_memory_limit $max_datafile_maxsize; then
            red_echo "${bold} The machine's minimum disk cannot be less than min_value (${double_memory_limit}G) (2 * memory_limit). Please try again. ${sgr0}"
            exit 1
        fi
        if compare_size ${datafile_maxsize} ${max_datafile_maxsize}; then
            red_echo "${bold} The maximum size of the data file cannot exceed the available resources (${max_datafile_maxsize}G). Please try again. ${sgr0}"
            flag=1
            continue
        elif compare_size ${double_memory_limit} ${datafile_maxsize}; then
            red_echo "${bold} The maximum size of the data file cannot be less than the minimum value (${double_memory_limit}G). Please try again. ${sgr0}"
            flag=1
            continue
        else
            datafile_maxsize=$(echo "$datafile_maxsize" | tr -cd '0-9.')
        fi
        log_disk_free=$(awk -v df="$disk_free" -v data_max="$datafile_maxsize" 'BEGIN { print df - data_max }')
        log_disk_free=$(printf "%.0f" "$log_disk_free")
        if [ "$log_disk_free" -lt 0 ]; then
            red_echo "${bold} The available disk space is not enough. Please try again. ${sgr0}"
            exit
        fi
        read -p "Enter the OB log disk size (Configurable Range[${double_memory_limit}, ${log_disk_free}], Default: ${default_log_disk_size}, Unit: G): " log_disk_size
        log_disk_size=${log_disk_size:-"${default_log_disk_size}"}
        if [ "$log_disk_size" -lt 0 ]; then
            red_echo "${bold} Invalid log disk size. Please try again. ${sgr0}"
            continue
        fi
        if compare_size $double_memory_limit $log_disk_free; then
            red_echo "${bold}  The machine's minimum disk space cannot be less than the minimum value (${double_memory_limit}G) (2 * memory_limit). Please try again. ${sgr0}"
            exit 1
        fi
        if compare_size ${log_disk_size} ${log_disk_free}; then
            red_echo "${bold} The size of the log disk cannot exceed the available resources (${log_disk_free}G). Please try again. ${sgr0}"
            flag=1
        elif compare_size ${double_memory_limit} ${log_disk_size}; then
            red_echo "${bold} The size of the log disk cannot be less than the min_value (${double_memory_limit}G). Please try again. ${sgr0}"
            flag=1
        else
            log_disk_size=$(echo "$log_disk_size" | tr -cd '0-9.')
        fi
        total_use=$(awk -v data_max="$datafile_maxsize" -v log_size="$log_disk_size" 'BEGIN { print data_max + log_size }')
        if compare_size ${total_use} ${all_disk_free}; then
            red_echo "${bold} The total usage cannot exceed the available resources (${disk_free}G). Please try again. ${sgr0}"
            flag=1
        fi
    else
        disk_free=$(printf "%.0f" "$disk_free")
        read -p "Enter the OB datafile maxsize (Configurable Range[${double_memory_limit}, ${disk_free}], Default: ${disk_free}, Unit: G): " datafile_maxsize
        datafile_maxsize=${datafile_maxsize:-"${disk_free}"}
        if [ "$datafile_maxsize" -lt 0 ]; then
            red_echo "${bold} Invalid datafile maxsize. Please try again. ${sgr0}"
            continue
        fi
        if compare_size $double_memory_limit $disk_free; then
            red_echo "${bold} The machine's minimum disk space cannot be less than the min_value (${double_memory_limit}G) (2 * memory_limit). Please try again. ${sgr0}"
            exit 1
        fi
        if compare_size ${datafile_maxsize} ${disk_free}; then
            red_echo "${bold} The maximum size of the data file cannot exceed the available resources (${disk_free}G). Please try again. ${sgr0}"
            flag=1
        else
            datafile_maxsize=$(echo "$datafile_maxsize" | tr -cd '0-9.')
        fi
        default_log_disk_size1=$(df -BG ${redo_dir} | awk 'NR==2 {print $4}')
        default_log_disk_size1=$(echo "$default_log_disk_size1" | tr -cd '0-9.')
        default_log_disk_size1=$(awk -v df="$default_log_disk_size1" 'BEGIN { print df * 0.9 }')
        default_log_disk_size1=$(printf "%.0f" "$default_log_disk_size1")
        default_log_disk_size2=$(awk -v a="$memory_limit" 'BEGIN{print a*2}')
        default_log_disk_size1=$(echo "$default_log_disk_size1" | tr -cd '0-9.')
        default_log_disk_size2=$(echo "$default_log_disk_size2" | tr -cd '0-9.')
        min_value=$( [[ "$default_log_disk_size1" -le "$default_log_disk_size2" ]] && echo "$default_log_disk_size1" || echo "$default_log_disk_size2" )
        min_value=$(printf "%.0f" "$min_value")
        default_log_disk_size1=$(printf "%.0f" "$default_log_disk_size1")
        read -p "Enter the OB log disk size (Configurable Range[${min_value}, ${default_log_disk_size1}], Default: ${default_log_disk_size1}, Unit: G): " log_disk_size
        log_disk_size=${log_disk_size:-"${default_log_disk_size1}"}
        if [ "$log_disk_size" -lt 0 ]; then
            red_echo "${bold} Invalid log disk size. Please try again. ${sgr0}"
            continue
        fi
        if compare_size $min_value $default_log_disk_size1; then
            red_echo "${bold} The machine's minimum disk space cannot be less than the min_value (${double_memory_limit}G) (2 * memory_limit). Please try again. ${sgr0}"
            exit 1
        fi
        if compare_size ${log_disk_size} ${default_log_disk_size1}; then
            red_echo "${bold} The size of the log disk cannot exceed the available resources (${default_log_disk_size1}G). Please try again. ${sgr0}"
            flag=1
        elif compare_size ${min_value} ${log_disk_size}; then
            red_echo "${bold} The size of the log disk cannot be less than the min_value (${min_value}G). Please try again. ${sgr0}"
            flag=1
        else
            log_disk_size=$(echo "$log_disk_size" | tr -cd '0-9.')
        fi
    fi
    if [[ $flag -eq 0 ]]; then
        break
    fi
done


while true; do
    read -p "${bold} Do you want to create tenant for your business workload? (y/n)(Default: y):  ${sgr0}" confirm3
    confirm3=${confirm3:-y}
    case $confirm3 in
        [Yy]* )
            sys_cpu=$(get_sys_cpu $cpu_count)
            sys_memory_size=1
            system_memory=$(get_system_memory $memory_limit)
            sys_log_disk_size=$(get_sys_log_disk_size $memory_limit)
            green_echo "${bold} Tenant: sys conifguration: ${sys_cpu}C/${sys_memory_size}G/${sys_log_disk_size}G(CPU/Memory/Log disk) ${sgr0}"
            green_echo "${bold} system_memory configuration(Unit: G): ${system_memory} ${sgr0}"
            # input tenant name
            while true; do
                read -p "Enter the tenant name (Default: test; allowed characters: letters, numbers, and underscores): " tenant_name
                tenant_name=${tenant_name:-test}
                if [[ ! "$tenant_name" =~ ^[a-zA-Z0-9_]+$ ]]; then
                    red_echo "${bold} The tenant name is invalid. Only letters, numbers, and underscores are allowed. ${sgr0}"
                else
                    break
                fi
            done

            # input tenant mode
            while true; do
                echo "${bold}    Please select the tenant mode (enter the corresponding number): ${sgr0}"
                echo "${bold}    1) MySQL${sgr0}"
                echo "${bold}    2) Oracle${sgr0}"
                read -p "Please enter your choice [1/2] (default 1):" tenant_mode
                tenant_mode=${tenant_mode:-"1"}
                if [[ "$tenant_mode" == "1" ]]; then
                    tenant_mode="mysql"
                elif [[ "$tenant_mode" == "2" ]]; then
                    tenant_mode="oracle"
                else
                    red_echo "${bold} The tenant mode is invalid. Only '1' and '2' are allowed. ${sgr0}"
                    continue
                fi
                break
            done

            # input tenant password
            while true; do
                if [ $tenant_mode == "mysql" ]; then
                    read -sp "Enter the tenant password: " tenant_password1
                    echo ""
                    read -sp "Confirm the tenant password: " tenant_password2
                    echo ""
                    if [[ "$tenant_password1" != "$tenant_password2" ]]; then
                        echo "${bold} The two passwords do not match. Please try again. ${sgr0}"
                        continue
                    fi
                fi
                break
            done

            # input tenant cpu
            while true; do
                default_tenant_cpu=$(awk -v a="$cpu_count" -v b="${sys_cpu}" 'BEGIN{print a - b}')
                read -p "Enter the tenant cpu (Configurable Range[1, ${default_tenant_cpu}], Default: ${default_tenant_cpu}): " tenant_cpu
                tenant_cpu=${tenant_cpu:-"${default_tenant_cpu}"}
                if [ "$tenant_cpu" -lt 0 ]; then
                    red_echo "${bold} Invalid tenant cpu. Please try again. ${sgr0}"
                    continue
                fi
                if compare_size 1 $default_tenant_cpu; then
                    red_echo "${bold} The cluster's minimum available CPU cannot be less than the min_value (1). Please try again. ${sgr0}"
                    exit 1
                fi
                if compare_size ${tenant_cpu} ${default_tenant_cpu}; then
                    red_echo "${bold} The tenant's CPU cannot exceed the cluster's available resources (${default_tenant_cpu}). Please try again. ${sgr0}"
                elif compare_size 1 ${tenant_cpu}; then
                    red_echo "${bold} The tenant's CPU cannot be less than 1. Please try again. ${sgr0}"
                else
                    break
                fi
            done

            # input tenant memory
            while true; do
                default_tenant_memory=$(awk -v a="$memory_limit" -v b="$system_memory" -v c="$sys_memory_size" 'BEGIN{print a - b - c}')
                default_tenant_memory=$(printf "%.0f" "$default_tenant_memory")
                read -p "Enter the tenant memory (Configurable Range[1, ${default_tenant_memory}], Default: ${default_tenant_memory}, Unit: G): " tenant_memory
                tenant_memory=${tenant_memory:-"${default_tenant_memory}"}
                if [ "$tenant_memory" -lt 0 ]; then
                    red_echo "${bold} Invalid tenant memory. Please try again. ${sgr0}"
                    continue
                fi
                if compare_size 1 $default_tenant_memory; then
                    red_echo "${bold} The cluster's minimum available memory cannot be less than the minimum value (1G). Please try again. ${sgr0}"
                    exit 1
                fi
                if compare_size ${tenant_memory} ${default_tenant_memory}; then
                    red_echo "${bold} The tenant's memory cannot exceed the cluster's available resources (${default_tenant_memory}G). Please try again. ${sgr0}"
                elif compare_size 1 ${tenant_memory}; then
                    red_echo "${bold} The tenant's memory cannot be less than 1G. Please try again. ${sgr0}"
                else
                    tenant_memory=$(echo "$tenant_memory" | tr -cd '0-9.')
                    break
                fi
            done

            # input tenant log disk size
            while true; do
                default_tenant_log_disk_size=$(awk -v a="$log_disk_size" -v b="$sys_log_disk_size" -v c="1" 'BEGIN{print a - b - c}')
                default_tenant_log_disk_size2=$(awk -v a="$tenant_memory" 'BEGIN{print a * 3}')
                default_tenant_log_disk_size2=$(echo "$default_tenant_log_disk_size2" | tr -cd '0-9.')
                min_value=$( [[ "$default_tenant_log_disk_size" -le "$default_tenant_log_disk_size2" ]] && echo "$default_tenant_log_disk_size" || echo "$default_tenant_log_disk_size2" )
                min_value=$(printf "%.0f" "$min_value")
                max_value=$default_tenant_log_disk_size
                if [[ $tenant_memory -ne ${default_tenant_memory} ]];then
                    default_tenant_log_disk_size=$default_tenant_log_disk_size2
                fi
                default_tenant_log_disk_size=$(printf "%.0f" "$default_tenant_log_disk_size")
                read -p "Enter the tenant log disk size(Configurable Range[${min_value}, ${max_value}], Default: ${default_tenant_log_disk_size}, Unit: G): " tenant_log_disk_size
                tenant_log_disk_size=${tenant_log_disk_size:-"${default_tenant_log_disk_size}"}
                if [ "$tenant_log_disk_size" -lt 0 ]; then
                    red_echo "${bold} Invalid tenant log disk size. Please try again. ${sgr0}"
                    continue
                fi
                if compare_size $min_value $max_value; then
                    red_echo "${bold} The cluster's minimum log disk size cannot be less than the min_value (${min_value}G). Please try again. ${sgr0}"
                    exit 1
                fi
                if compare_size ${tenant_log_disk_size} ${default_tenant_log_disk_size}; then
                    red_echo "${bold} The tenant's log disk size cannot exceed the cluster's available resources (${default_tenant_log_disk_size}G). Please try again. ${sgr0}"
                elif compare_size ${min_value} ${tenant_log_disk_size}; then
                    red_echo "${bold} The tenant's log disk size cannot be less than the min_value (${min_value}G). Please try again. ${sgr0}"
                else
                    tenant_log_disk_size=$(echo "$tenant_log_disk_size" | tr -cd '0-9.')
                    break
                fi
            done
            break;;
        [Nn]* ) echo "OceanBase Database will not be create tenant.";break;;
        * ) echo "Please answer y or n.";;
    esac
done

while true; do
    read -p "${bold} Do you want to enable encryption for password security? (y/n) (Default: n):  ${sgr0}" confirm5
    confirm5=${confirm5:-n}
    case $confirm5 in
      [Yy]* ) obd pwd encrypt enable; break;;
      [Nn]* ) break;;
      * ) echo "Please answer y or n.";;
    esac
done


while true; do
    read -p "${bold} Do you need to install the monitoring components (OBAgent, Prometheus, Grafana)? (y/n) (Default: y):  ${sgr0}" confirm1
    confirm1=${confirm1:-y}
    case $confirm1 in
        [Yy]* )
            # input obagent monitoring service port
            while true; do
                read -p "Enter the OBAgent monitoring service port (Default: 8088): " monagent_http_port
                monagent_http_port=${monagent_http_port:-8088}
                if port_check ${monagent_http_port}; then
                    break
                fi
            done

            # input obagent management service port
            while true; do
                read -p "Enter the OBAgent management service port (Default: 8089): " mgragent_http_port
                mgragent_http_port=${mgragent_http_port:-8089}
                if port_check ${mgragent_http_port}; then
                    break
                fi
            done

            # input prometheus port
            while true; do
                read -p "Enter the Prometheus port (Default: 9090): " prometheus_port
                prometheus_port=${prometheus_port:-9090}
                if port_check ${prometheus_port}; then
                    break
                fi
            done

            # input grafana port
            while true; do
                read -p "Enter the Grafana port (Default: 3000): " grafana_port
                grafana_port=${grafana_port:-3000}
                if port_check ${grafana_port}; then
                    break
                fi
            done
            break;;
        [Nn]* ) echo "Monitoring Component will not be installed."; break;;
        * ) echo "Please answer y or n.";;
    esac
done


# print configurations
echo ""
echo "#Saved configurations: "
echo "${bold} cluster name: ${cluster_name} ${sgr0}"
echo "${bold} mysql port: ${mysql_port} ${sgr0}"
echo "${bold} rpc port: ${rpc_port} ${sgr0}"
echo "${bold} obshell port: ${obshell_port} ${sgr0}"
echo "${bold} cpu count: ${cpu_count} ${sgr0}"
echo "${bold} memory limit: $(format_type $memory_limit) ${sgr0}"
echo "${bold} home path: ${home_path} ${sgr0}"
echo "${bold} data dir: ${data_dir} ${sgr0}"
echo "${bold} log dir: ${redo_dir} ${sgr0}"
echo "${bold} datafile maxsize: $(format_type $datafile_maxsize) ${sgr0}"
echo "${bold} log disk size: $(format_type $log_disk_size) ${sgr0}"
if [ ${confirm3} = y ]; then
    echo "${bold} tenant name: ${tenant_name} ${sgr0}"
    echo "${bold} tenant cpu: ${tenant_cpu} ${sgr0}"
    echo "${bold} tenant memory: $(format_type $tenant_memory) ${sgr0}"
    echo "${bold} tenant log disk size: $(format_type $tenant_log_disk_size) ${sgr0}"
fi

if [ ${confirm1} = y ]; then
    echo "${bold} OBAgent monitoring service port: ${monagent_http_port} ${sgr0}"
    echo "${bold} OBAgent management service port: ${mgragent_http_port} ${sgr0}"
    echo "${bold} Prometheus port: ${prometheus_port} ${sgr0}"
    echo "${bold} Grafana port: ${grafana_port} ${sgr0}"
fi

echo ""
# 最终确认
while true; do
    read -p "${bold} Are you sure these configurations are correct? (y/n) (Default: y):  ${sgr0}" confirm2
    confirm2=${confirm2:-y}
    case $confirm2 in
        [Yy]* ) echo "Configuration confirmed."; break;;
        [Nn]* ) echo "Please execute the script again to input the information."; exit;;
        * ) echo "Please answer y or n.";;
    esac
done
echo ""

random_number=$RANDOM
config=auto_config_$random_number.yaml
cat > ${config} <<EOF
user:
  username: ${username}
  password: ${user_password}
  port: 22
oceanbase-standalone:
  servers:
    - ${ip}
  global:
    cluster_name: ${cluster_name}
    mysql_port: ${mysql_port}
    rpc_port: ${rpc_port}
    obshell_port: ${obshell_port}
    root_password: ${root_password}
    home_path: ${home_path}/oceanbase
    data_dir: ${data_dir}
    redo_dir: ${redo_dir}
    cpu_count: ${cpu_count}
    memory_limit: $(format_type $memory_limit)
    datafile_size: 2G
    datafile_maxsize: $(format_type $datafile_maxsize)
    log_disk_size: $(format_type $log_disk_size)
    system_memory: 0
    __min_full_resource_pool_memory: 1073741824
    zone: zone1
EOF

if [ ${confirm1} = y ]; then
    cat >> ${config} <<EOF
obagent:
  depends:
    - oceanbase-standalone
  servers:
    - ${ip}
  global:
    home_path: ${home_path}/obagent
    monagent_http_port: ${monagent_http_port}
    mgragent_http_port: ${mgragent_http_port}
prometheus:
  depends:
    - obagent
  servers:
    - ${ip}
  global:
    home_path: ${home_path}/prometheus
    port: ${prometheus_port}
grafana:
  depends:
    - prometheus
  servers:
    - ${ip}
  global:
    home_path: ${home_path}/grafana
    port: ${grafana_port}
    login_password: oceanbase
EOF
fi

echo ""
echo "start cluster..."
obd cluster autodeploy ${cluster_name} -c ${config} -f
if [[ $? -ne 0 ]]; then
    echo "start cluster failed"
    exit 1
fi

if [ ${confirm3} = y ]; then
    echo ""
    echo "create tenant..."
    tenant_memory=$(format_type $tenant_memory)
    tenant_log_disk_size=$(format_type $tenant_log_disk_size)
    if [ ${tenant_mode} = "mysql" ]; then
        obd cluster tenant create ${cluster_name} -t ${tenant_name} --memory-size=${tenant_memory} --max-cpu=${tenant_cpu} --log-disk-size=${tenant_log_disk_size} --password=${tenant_password2}  --primary-zone=zone1
    else
        obd cluster tenant create ${cluster_name} -t ${tenant_name} --memory-size=${tenant_memory} --max-cpu=${tenant_cpu} --log-disk-size=${tenant_log_disk_size} --mode=${tenant_mode} --primary-zone=zone1
    fi

    if [[ $? -ne 0 ]]; then
        echo "create tenant failed"
        exit 1
    fi

    echo ""
    echo "You can access the ${tenant_name} tenant using the following command.："
    if [ ${tenant_mode} = "mysql" ]; then
        green_echo "${bold} obclient -h${ip} -P${mysql_port} -uroot@${tenant_name} -p${tenant_password2} ${sgr0}"
    else
        echo "${bold} For security reasons, kindly reset the root password immeditely once the tenant has been successfully created. ${sgr0}"
        green_echo "${bold} obclient -h${ip} -P${mysql_port} -uSYS@${tenant_name} ${sgr0}"
    fi
fi

yellow_echo "If this cluster is for production use, please import a commercial license in time."